@@ -1,18 +1,99 @@
+"as.matrix.partition" <- function(x, ...){
+  class(x) <- "matrix"
+  NextMethod("as.matrix")
+}
+
+"summary.partition" <- function(object, ...){
+  jj <- list(...)
+  if(length(jj)>0){
+    if(!is.null(jj$n)){
+      n <- jj$n
+    } else {
+      n <- jj[[1]]
+    }
+  } else {
+    n <- 10
+  }
+  if(ncol(object)>2*n){
+    jj <- cbind(object[,seq_len(n)],object[,ncol(object)+1-seq_len(n)])
+    shortened <- TRUE
+  } else {
+    jj <- object
+    shortened <- FALSE
+  }
+  out <- list(shortened=shortened, n=n, out=jj)
+  class(out) <- "summary.partition"
+  return(out)
+}
+
+print.summary.partition <- function(x, ...){
+  if(x$shortened){
+    jj <- seq_len(x$n)
+    x <- x$out
+    x <- cbind(x[,jj],"...",x[,ncol(x)+1-jj])
+  } else {
+    x <- x$out
+  }
+  print.partition(x)
+}
+  
+"setparts" <- function(x){
+  if(length(x)==1){return(Recall(parts(x)))}
+  if(is.matrix(x)){
+    out <- do.call("cbind",apply(x,2,setparts))
+  } else {
+    x <- sort(x[x>0], decreasing=TRUE)
+    num.of.parts <-
+      factorial(sum(x))/(prod(c(factorial(x),factorial(table(x)))))
+    out <- .C("wrap",
+              as.integer(x),
+              as.integer(length(x)),
+              ans = integer(sum(x)*num.of.parts),
+              PACKAGE="partitions"
+              )$ans
+    dim(out) <- c(sum(x),num.of.parts)
+  }
+  class(out) <- c("set_partition","partition")
+  return(out)
+}
+
+"print.partition" <- function(x, mat=getOption("matrixlike"), h=getOption("horiz"), ...){
+  class(x) <- "matrix"
+  if(!isTRUE(mat)){
+    colnames(x) <- rep(" ", ncol(x))
+  }
+  if(isTRUE(h)){
+    x <- t(x)
+  }
+  return(invisible(print(noquote(x))))
+}
+
 "parts" <-
 function(n){
-  nn <- P(n)
-  out <- .C("allparts",
-           as.integer(n),
-           as.integer(n*nn),
-           ans = integer(n*nn),
-           PACKAGE="partitions"
-           )$ans
-  dim(out) <- c(n,nn)
+  if(length(n)>1){
+    stop("argument must be of length 1")
+  }
+  if(n>0){
+    nn <- P(n)
+    out <- .C("allparts",
+              as.integer(n),
+              as.integer(n*nn),
+              ans = integer(n*nn),
+              PACKAGE="partitions"
+              )$ans
+    dim(out) <- c(n,nn)
+  } else {
+    out <- matrix(0,0,0)
+  }
+  class(out) <- c("integer_partition","partition")
   return(out)
 }
 
 "diffparts" <-
 function(n){
+  if(length(n)>1){
+    stop("argument must be of length 1")
+  }
   nn <- Q(n)
   n.tri <- floor( (sqrt(1+8*n)-1)/2)
   out <- .C("alldiffparts",
@@ -23,6 +104,7 @@ function(n){
             PACKAGE="partitions"
             )$ans[-nn*n.tri-1]
     dim(out) <- c(n.tri,nn)
+    class(out) <- c("diff_integer_partition","partition")
   return(out)
 }
 
@@ -30,23 +112,23 @@ function(n){
   jj.n <- R(m,n,include.zero=include.zero)
   len <- m*jj.n
 
-  jj <- .C("allrestrictedparts",
+  out <- .C("allrestrictedparts",
            as.integer(m),
            as.integer(n),
            as.integer(len),
            as.integer(include.zero),
            ans=integer(len),
            PACKAGE="partitions"
-           )
-  out <- jj$ans
+           )$ans
   dim(out) <- c(m,jj.n)
   if(decreasing){
-    return(out[m:1,])
-  } else {
-    return(out)
+    out <- out[rev(seq_len(m)),]
   }
+  class(out) <- c("restrictedpartition","partition")
+  return(out)
 }
 
+
 "blockparts" <- function(f,n=NULL,include.fewer=FALSE){
   s <- sum(f)
   if(is.null(n)){
@@ -74,18 +156,25 @@ function(n){
     out <- replace(matrix(0,lf,ncol(out)),f>0,out)
   }
   rownames(out) <- nf
-  colnames(out) <- rep(" ",nb)
+  class(out) <- c("block_partition","partition")
   return(out)
 }
     
-"allparts" <-
-function(n){
+"compositions" <-
+function(n, m=NULL, include.zero=TRUE){
+    if(!is.null(m)){
+      if(include.zero){
+        return(blockparts(rep(n,m),n))
+      } else {
+        return(1L + blockparts(rep(n-1,m),n))
+      }
+    }
     pad <- function(i,n){c(i,integer(n-length(i)))}
     f <- function(x){diff(c(as.integer(0),which(c(x,TRUE))))}
     jj <- apply(as.matrix(expand.grid(rep(list(c(TRUE,FALSE)),n-1))),1, f)
     out <-  do.call(cbind,lapply(jj, pad, n))
     rownames(out) <- NULL
-    colnames(out) <- rep(" ",ncol(out))
+    class(out) <- c("all_partitions","partition")
     return(out)
 }
 
@@ -163,6 +252,9 @@ function(n, give=FALSE){
            PACKAGE = "partitions"
            )$ans
   dim(out) <- c(mx,nc)
+  if(nc>1){
+    class(out) <- c("conjugate_partition","partition")
+  }
   return(drop(out))
 }
 
